@@ -11,16 +11,22 @@ import sys
 import tempfile
 import urllib
 import uuid
+import concurrent.futures
 
-import youtube_dl
 from bs4 import BeautifulSoup
 from kiwixstorage import KiwixStorage
 from pif import get_public_ip
 from slugify import slugify
-from zimscraperlib.download import save_large_file
-from zimscraperlib.imaging import resize_image, convert_image
+from zimscraperlib.download import save_large_file, YoutubeDownloader, BestMp4, BestWebm
+from zimscraperlib.image.transformation import resize_image
+from zimscraperlib.image.convertion import convert_image
 from zimscraperlib.video.encoding import reencode
-from zimscraperlib.video.presets import VideoMp4Low, VideoWebmLow
+from zimscraperlib.video.presets import (
+    VideoMp4Low,
+    VideoWebmLow,
+    VideoMp4High,
+    VideoWebmHigh,
+)
 from zimscraperlib.zim import make_zim_file
 
 from .annex import MoocForum, MoocWiki
@@ -109,6 +115,7 @@ class Openedx2Zim:
         no_zim,
         keep_build_dir,
         debug,
+        threads,
     ):
 
         # video-encoding info
@@ -139,6 +146,8 @@ class Openedx2Zim:
         self.ignore_missing_xblocks = ignore_missing_xblocks
         self.autoplay = autoplay
         self.remove_seq_nav = remove_seq_nav
+        self.threads = threads
+        self.yt_downloader = YoutubeDownloader(threads=1)
 
         # authentication
         self.email = email
@@ -355,10 +364,10 @@ class Openedx2Zim:
 
         # check for paths in org_tab_path
         if tab_org_path == "course" or "courseware" in tab_org_path:
-            tab_name = tab_text.replace(", current location", "")
+            tab_name = "Course"
             tab_path = "course/" + self.head_course_xblock.folder_name + "/index.html"
         elif "info" in tab_org_path:
-            tab_name = tab_text.replace(", current location", "")
+            tab_name = "Course Info"
             tab_path = "/index.html"
         elif "wiki" in tab_org_path and self.add_wiki:
             self.wiki = MoocWiki(self)
@@ -456,7 +465,7 @@ class Openedx2Zim:
         save_large_file(self.favicon_url, favicon_fpath)
 
         # convert and resize
-        convert_image(favicon_fpath, favicon_fpath, "PNG")
+        convert_image(favicon_fpath, favicon_fpath, fmt="PNG")
         resize_image(favicon_fpath, 48, allow_upscaling=True)
 
         if not favicon_fpath.exists():
@@ -525,7 +534,14 @@ class Openedx2Zim:
 
         # make xblock_extractor objects download their content
         logger.info("Getting content for supported xblocks ...")
-        self.head_course_xblock.download(self.instance_connection)
+        with concurrent.futures.ThreadPoolExecutor(
+            max_workers=self.threads
+        ) as executor:
+            fs = [
+                executor.submit(xblock.download, self.instance_connection)
+                for xblock in self.xblock_extractor_objects
+            ]
+            concurrent.futures.wait(fs, return_when=concurrent.futures.ALL_COMPLETED)
 
     def s3_credentials_ok(self):
         logger.info("Testing S3 Optimization Cache credentials ...")
@@ -600,34 +616,35 @@ class Openedx2Zim:
             return None
 
     def download_from_youtube(self, url, fpath):
-        audext, vidext = {"webm": ("webm", "webm"), "mp4": ("m4a", "mp4")}[
-            self.video_format
-        ]
         output_file_name = fpath.name.replace(fpath.suffix, "")
-        options = {
-            "outtmpl": str(fpath.parent.joinpath(f"{output_file_name}.%(ext)s")),
-            "preferredcodec": self.video_format,
-            "format": f"best[ext={vidext}]/bestvideo[ext={vidext}]+bestaudio[ext={audext}]/best",
-            "retries": 20,
-            "fragment-retries": 50,
-        }
+        options = (BestWebm if self.video_format == "webm" else BestMp4).get_options(
+            target_dir=fpath.parent,
+            filepath=pathlib.Path(f"{output_file_name}.%(ext)s"),
+            format=f"best[ext={self.video_format}]/best",
+        )
         try:
-            with youtube_dl.YoutubeDL(options) as ydl:
-                ydl.download([url])
-                for content in fpath.parent.iterdir():
-                    if content.is_file() and content.name.startswith(
-                        f"{output_file_name}."
-                    ):
-                        return content
+            self.yt_downloader.download(url, options)
+            for content in fpath.parent.iterdir():
+                if content.is_file() and content.name.startswith(
+                    f"{output_file_name}."
+                ):
+                    return content
         except Exception as exc:
-            logger.error(f"Error while running youtube_dl: {exc}")
+            logger.error(f"Error while downloading from youtube: {exc}")
             return None
 
     def convert_video(self, src, dst):
-        if (src.suffix[1:] != self.video_format) or self.low_quality:
+        preset = None
+        if self.low_quality:
             preset = VideoWebmLow() if self.video_format == "webm" else VideoMp4Low()
+        elif src.suffix[1:] != self.video_format:
+            preset = VideoWebmHigh() if self.video_format == "webm" else VideoMp4High()
             return reencode(
-                src, dst, preset.to_ffmpeg_args(), delete_src=True, failsafe=False,
+                src,
+                dst,
+                preset.to_ffmpeg_args(),
+                delete_src=True,
+                failsafe=False,
             )
 
     def optimize_image(self, src, dst):
@@ -666,8 +683,8 @@ class Openedx2Zim:
         return f"{fpath.suffix[1:]}/{safe_url}/{quality}"
 
     def download_file(self, url, fpath):
-        """ downloads a file from the supplied url to the supplied fpath
-            returns true if successful, false if unsuccessful """
+        """downloads a file from the supplied url to the supplied fpath
+        returns true if successful, false if unsuccessful"""
 
         is_youtube = "youtube" in url
         downloaded_from_cache = False
@@ -801,7 +818,9 @@ class Openedx2Zim:
         self.instance_config.update({"instance_url": f"https://{instance_netloc}"})
         logger.info("Testing openedx instance credentials ...")
         self.instance_connection = InstanceConnection(
-            self.email, self.password, self.instance_config,
+            self.email,
+            self.password,
+            self.instance_config,
         )
         self.instance_connection.establish_connection()
         jinja_init()
@@ -838,4 +857,6 @@ class Openedx2Zim:
             if not self.keep_build_dir:
                 logger.info("Removing temp folder...")
                 shutil.rmtree(self.build_dir, ignore_errors=True)
+        # shutdown the youtube downloader
+        self.yt_downloader.shutdown()
         logger.info("Done everything")
