@@ -13,10 +13,6 @@ class Course(BaseXblock):
             xblock_json, relative_path, root_url, xblock_id, descendants, scraper
         )
 
-    def download(self, instance_connection):
-        for x in self.descendants:
-            x.download(instance_connection)
-
     def render(self):
         for x in range(0, len(self.descendants)):
             if x == 0:
